package bfv.hardware;

import bfv.desktop.BFVDesktop;

import javax.swing.*;
import java.util.ArrayList;

/**
 * Created by AlFly on 14/11/15.
 */
public class HardwareCommands {

    ArrayList<HardwareCommand> commands;


    BFVDesktop desktop;

    public HardwareCommands(BFVDesktop desktop) {

        this.setupCommands();
        this.desktop = desktop;
    }

    public ArrayList<HardwareCommand> getCommands() {
        return commands;
    }

    public void setupCommands() {

        commands = new ArrayList<HardwareCommand>();

        commands.add(new HardwareCommand("BVU", "Volume Up (x2)"));
        commands.add(new HardwareCommand("BVD", "Volume Down (/2)"));
        commands.add(new HardwareCommand("BST", "Get Settings"));
        commands.add(new HardwareCommand("TMP", "Get Temperature"));
        commands.add(new HardwareCommand("RST", "Simple Reset"));
        commands.add(new HardwareCommand("RSX", "Reset and restore default settings"));
        commands.add(new HardwareCommand("SLP", "Sleep"));
        commands.add(new HardwareCommand("SLX", "Sleep (no UART wake)"));
        commands.add(new HardwareCommand("BTN", "Simulate Button"));

        HardwareCommand cmd = new HardwareCommand("BSD", "Play Sound");
        cmd.setParams("800 500 400 500");
        commands.add(cmd);

        cmd = new HardwareCommand("PMTK184,1", "Erase Locus");
        cmd.setAddChecksum(true);
        cmd.setAddCRLF(true);
        commands.add(cmd);

        cmd = new HardwareCommand("PMTK183", "Query Locus");
        cmd.setAddChecksum(true);
        cmd.setAddCRLF(true);
        commands.add(cmd);

        cmd = new HardwareCommand("PMTK622,0", "Query Locus Data");
        cmd.setAddChecksum(true);
        cmd.setAddCRLF(true);
        commands.add(cmd);
    }



//    public HardwareCommand getCommand(String code) {
//        if (commands != null) {
//            for (int i = 0; i < commands.size(); i++) {
//                HardwareCommand parameter = commands.get(i);
//                if (parameter.getCode().equals(code)) {
//                    return parameter;
//                }
//            }
//        }
//        return null;
//    }
//
//    public String[] getCommandLabels() {
//        if (commands != null) {
//            String[] ret = new String[commands.size()];
//            for (int i = 0; i < commands.size(); i++) {
//                ret[i] = commands.get(i).getMessage();
//            }
//            return ret;
//        }
//        return new String[]{"null"};
//    }


    public void sendCommand(HardwareCommand command) {
        String code = command.getCode();

        String message = "";
        if(command.getParams() != null){
            message ="$" + code + " " + command.getParams() + "*";
        }
        else{
            message = "$" + code + "*";
        }
        if(command.isAddChecksum()){
            message = message + Integer.toHexString(checksum(code)).toUpperCase();
        }

        if(command.isAddCRLF()){
            message = message + "\r\n";
        }
        desktop.sendOutput(message,true);

//        SwingUtilities.invokeLater(new Runnable() {
//            public void run() {
//                try {
//                    Thread.sleep(50);
//                } catch (InterruptedException e) {
//                    e.printStackTrace();
//                }
//                desktop.getParametersFromDevice();
//            }
//        });

    }

    public static int checksum(String s) {

        char[] msgArray = s.toCharArray();

        int checksum = 0;
        for ( char c : msgArray )
        {
            if ( (c == '!') || (c == '$')){
                return -1;
            }


            if (c == '*')
                break;

            checksum ^= c;
        }
        return checksum;

    }

}
