package circle;

import java.awt.image.BufferedImage;
import java.lang.*;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.io.*;
import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;


public class TaubinTest extends JPanel implements Runnable {
    private int size;

    private int x = 0;
    private double[][] points;
    private double[][] newPoints;

    private double[] result;
    private double[] result2;
    private double[] result3;
    private double[] errors;
    private double[] errors3;
    private int index = 0;
    private int time = 0;
    private boolean firstLot = false;
    private  WindCalculator windCalculator = new WindCalculator(50,0.2, 300);

    public TaubinTest(int size) {
        this.size = size;
        points = new double[size][2];
        result = new double[3];
        result2 = new double[3];
        errors = new double[3];
        new Thread(this).start();

    }

    public void paintComponent(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;

        this.addImageGraphics(g2);

    }

    public void addImageGraphics(Graphics2D g2) {
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g2.setColor(Color.WHITE);
        g2.fillRect(0, 0, 640, 480);

        g2.setColor(Color.BLACK);
        g2.drawString("Time " + time + "s", 20, 20);
        g2.drawString("Wind Speed " + new DecimalFormat("0.00").format(Math.sqrt(result2[0] * result2[0] + result2[1] * result2[1])) + "m/s", 20, 40);
        g2.drawString("Wind Direction " + new DecimalFormat("0").format(resolveDegrees(Math.toDegrees(Math.atan2(result2[1], result2[0])) - 90.0)) + "deg", 20, 60);

        g2.translate(200, 200);
        g2.scale(1.0, -1.0); //flip y
        // g2.scale(10,10);
        double scale = 10.0;
        g2.setColor(Color.RED);
        g2.draw(new CenteredEllipse(0, 0, 2, 2));
        g2.setColor(Color.BLACK);
        for (int i = 0; i < size; i++) {
            g2.draw(new CenteredEllipse(points[i][0] * scale, points[i][1] * scale, 1, 1));

        }


        for (int i = 0; i < newPoints.length; i++) {
            double[] point = newPoints[i];
            Line2D.Double line = new Line2D.Double(0,0,point[0] * scale, point[1] * scale);
            g2.draw(line);
        }


        if (firstLot) {


            //        g2.setColor(Color.BLUE);
            //        g2.draw(new Ellipse2D.Double((result[0] - result[2]) * scale, (result[1] - result[2]) * scale, result[2] * 2.0 * scale , result[2] * 2.0 * scale));
            g2.setColor(Color.MAGENTA);


            double x = result2[0] * scale;
            double y = result2[1] * scale;
            double r = result2[2] * scale;
            //center mse ellipse
            g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
            g2.fill(new CenteredEllipse(x, y, errors[2] * scale, errors[2] * scale));
            g2.draw(new Line2D.Double(0, 0, x, y));


            g2.setColor(Color.BLUE);
            // g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.0f));
            g2.draw(new CenteredEllipse(x, y, r - errors[0] * scale, r - errors[1] * scale));
            g2.draw(new CenteredEllipse(x, y, r + errors[0] * scale, r + errors[1] * scale));
        }

        if (newPoints.length > 2) {


                    //        g2.setColor(Color.BLUE);
                    //        g2.draw(new Ellipse2D.Double((result[0] - result[2]) * scale, (result[1] - result[2]) * scale, result[2] * 2.0 * scale , result[2] * 2.0 * scale));
                    g2.setColor(Color.YELLOW);


                    double x = result3[0] * scale;
                    double y = result3[1] * scale;
                    double r = result3[2] * scale;
                    //center mse ellipse
                    g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
                    g2.fill(new CenteredEllipse(x, y, errors3[2] * scale, errors3[2] * scale));
                    g2.draw(new Line2D.Double(0, 0, x, y));


                    g2.setColor(Color.GREEN);
                    // g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.0f));
                    g2.draw(new CenteredEllipse(x, y, r - errors3[0] * scale, r - errors3[1] * scale));
                    g2.draw(new CenteredEllipse(x, y, r + errors3[0] * scale, r + errors3[1] * scale));
                }

    }

    public void drawImage() {
        BufferedImage bi = new BufferedImage(640, 480, BufferedImage.TYPE_INT_ARGB);
        File outputfile = new File("output/Taubin" + time + ".png");
        Graphics2D g = bi.createGraphics();
        this.addImageGraphics(g);

        try {
            ImageIO.write(bi, "png", outputfile);
        } catch (IOException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    public void run() {


        try {

            BufferedReader in = new BufferedReader(new FileReader("in.txt"));

            String line ;
            double time = 0;

            while ((line = in.readLine()) != null) {
                String[] split = line.split(",");

                double x = Double.parseDouble(split[0]);
                double y = Double.parseDouble(split[1]);
                double bearing =resolveDegrees(Math.toDegrees(Math.atan2(x, y)));

                double speed = Math.sqrt(x * x + y * y);

               // System.out.println(x +","+y+","+bearing+","+speed + "," + Math.sin(Math.toRadians(bearing)) * speed + "," + Math.cos(Math.toRadians(bearing)) * speed);
                time += 1.0;
                windCalculator.addSpeedVector(bearing, speed, time);
                newPoints = windCalculator.getPoints();

                points[index][0] = x;
                points[index][1] = y;
                index++;
                time++;
                if (index == size) {
                    firstLot = true;
                    index = 0;
                }
                // if(firstLot){

                Thread.sleep(10);
                // result = FitCircle.kasaFit(points);
                result2 = FitCircle.taubinNewton(points);
                errors = FitCircle.getErrors(points, result2);

                if(newPoints.length > 2){
                    result3 = FitCircle.taubinNewton(newPoints);
                    errors3 = FitCircle.getErrors(newPoints, result3);
                    System.out.println(result2[0] + " , " + result2[1] + " , " + result2[2] + " , " +result3[0] + " , " + result3[1] + " , " + result3[2] + " , ");
                }



//                System.out.println(result2[0] + " , " + result2[1] + " , " + result2[2] + " , " +result3[0] + " , " + result3[1] + " , " + result3[2] + " , ");


//                this.repaint();
//                this.drawImage();
//
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void main(String[] args) {


        JFrame frame = new JFrame();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        TaubinTest panel = new TaubinTest(90);
        panel.setPreferredSize(new Dimension(640, 480));
        frame.add(panel);
        frame.setLocationRelativeTo(null);
        frame.pack();
        frame.show();
    }

    public double resolveDegrees(double degrees) {
        if (degrees < 0.0) {
            return resolveDegrees(degrees + 360.0);

        }
        if (degrees > 360.0) {
            return resolveDegrees(degrees - 360.0);
        }
        return degrees;
    }


}